@@ -2,13 +2,13 @@
 // Program to pull the information out of various types of EXIF digital 
 // camera files and show it in a reasonably consistent way
 //
-// Version 2.2
+// Version 2.3
 //
 //
-// Compiling under Windows:  Use microsoft's compile.  from command line:
+// Compiling under Windows:  Use microsoft's compiler.  from command line:
 // cl -Ox jhead.c exif.c myglob.c
 //
-// Dec 1999 - June 2004
+// Dec 1999 - Jan 2005
 //
 // by Matthias Wandel   www.sentex.net/~mwandel
 //--------------------------------------------------------------------------
@@ -22,7 +22,7 @@
 #include <errno.h>
 #include <ctype.h>
 
-#define JHEAD_VERSION "2.2"
+#define JHEAD_VERSION "2.3-2"
 
 // This #define turns on features that are too very specific to 
 // how I organize my photos.  Best to ignore everything inside #ifdef MATTHIAS
@@ -66,6 +66,7 @@ static time_t ExifTimeSet = 0;      // Set exif time to a value.
 
 static int DeleteComments = FALSE;
 static int DeleteExif = FALSE;
+static int DeleteUnknown = FALSE;
 static char * ThumbnailName = NULL; // If not NULL, use this string to make up
                                     // the filename to store the thumbnail to.
 
@@ -288,7 +289,7 @@ static int AutoResizeCmdStuff(void)
 
     if (scale > 0.8) return FALSE; // Don't rescale by really small amounts (not worth it!)
 
-    sprintf(CommandString, "mogrify -geometry %dx%d -quality 80 &i",(int)(ImageInfo.Width*scale), (int)(ImageInfo.Height*scale));
+    sprintf(CommandString, "mogrify -geometry %dx%d -quality 82 &i",(int)(ImageInfo.Width*scale), (int)(ImageInfo.Height*scale));
     return TRUE;
 }
 
@@ -447,8 +448,7 @@ void DoFileRenaming(const char * FileName)
         }
     }
 
-
-    if (!ImageInfo.DateTime[0] || !Exif2tm(&tm, ImageInfo.DateTime)){
+    if (ImageInfo.numDateTimeTags == 0 || !Exif2tm(&tm, ImageInfo.DateTimePointers[0])){
         printf("File '%s' contains no exif date stamp.  Using file date\n",FileName);
         // Use file date/time instead.
         tm = *localtime(&ImageInfo.FileDateTime);
@@ -493,6 +493,7 @@ void DoFileRenaming(const char * FileName)
                         char pat[8];
                         char num[16];
                         memcpy(pat, pattern+ppos, 4);
+                        pat[a-ppos] = 'd'; // Replace 'i' with 'd' for '%d'
                         pat[a-ppos+1] = '\0';
                         sprintf(num, pat, FilesMatched); // let printf do the number formatting.
                         memmove(pattern+ppos+strlen(num), pattern+a+1, strlen(pattern+a+1)+1);
@@ -563,12 +564,22 @@ void ProcessFile(const char * FileName)
     int Modified = FALSE;
     ReadMode_t ReadMode = READ_EXIF;
     CurrentFile = FileName;
+    FilesMatched += 1; // Count files processed.
+
+    if (DoModify || RenameToDate || Exif2FileTime){
+        if (access(FileName, 2 /*W_OK*/)){
+            printf("Skipping readonly file '%s'\n",FileName);
+            return;
+        }
+    }
+
     ResetJpgfile();
 
     // Start with an empty image information structure.
     memset(&ImageInfo, 0, sizeof(ImageInfo));
     ImageInfo.FlashUsed = -1;
     ImageInfo.MeteringMode = -1;
+    ImageInfo.Whitebalance = -1;
 
     // Store file date/time.
     {
@@ -645,8 +656,6 @@ void ProcessFile(const char * FileName)
         ReadMode = READ_IMAGE;
     }
 
-    FilesMatched += 1; // Count files processed.
-
     if (DoModify){
         ReadMode |= READ_IMAGE;
     }
@@ -862,6 +871,9 @@ void ProcessFile(const char * FileName)
     if (DeleteExif){
         if (RemoveSectionType(M_EXIF)) Modified = TRUE;
     }
+    if (DeleteUnknown){
+        if (RemoveUnknownSections()) Modified = TRUE;
+    }
 
 
     if (Modified){
@@ -887,12 +899,12 @@ void ProcessFile(const char * FileName)
 
     if (Exif2FileTime){
         // Set the file date to the date from the exif header.
-        if (ImageInfo.DateTime[0]){
+        if (ImageInfo.numDateTimeTags){
             // Converte the file date to Unix time.
             struct tm tm;
             time_t UnixTime;
             struct utimbuf mtime;
-            if (!Exif2tm(&tm, ImageInfo.DateTime)) goto badtime;
+            if (!Exif2tm(&tm, ImageInfo.DateTimePointers[0])) goto badtime;
 
             UnixTime = mktime(&tm);
             if ((int)UnixTime == -1){
@@ -939,15 +951,16 @@ static void Usage (void)
     printf("Where:\n"
            "[options] are:\n"
            "  -dc        Delete comment field (as left by progs like Photoshop & Compupic)\n"
+           "  -de        Strip Exif section (smaller JPEG file, but lose digicam info)\n"
+           "  -du        Delete non image sections except for Exif and comment sections\n"
+           "  -purejpg   Strip all unnecessary data from jpeg (combines -dc -de and -du)\n"
            "  -ce        Edit comment field.  Uses environment variable 'editor' to\n"
            "             determine which editor to use.  If editor not set, uses VI\n"
            "             under Unix and notepad with windows\n"
-
            "  -cs <name> Save comment section to a file\n"
            "  -ci <name> Insert comment section from a file.  -cs and -ci use same naming\n"
            "             scheme as used by the -st option\n"
            "  -cl string Insert literal comment stirng\n"
-           "  -de        Strip Exif section (smaller JPEG file, but lose digicam info)\n"
            "  -autorot   Invoke jpegtran to rotate images according to Exif orientation tag\n"
            "             Note: Windows users must get jpegtran for this to work\n"
            "  -norot     Zero out the rotation tag.  This to avoid some browsers from\n" 
@@ -988,12 +1001,12 @@ static void Usage (void)
            "             '%%f' as part of the string will include the original file name\n"
            "             '%%i' will include a sequence nubmer, starting from 1. You can\n"
            "             You can specify '%%03i' for example to get leading zeros.\n"
-           "             This feature is useful for ordering files from multipe digicams to\n"
+           "             This feature is useful for ordering files from multiple digicams to\n"
            "             sequence of taking.  Only renames files whose names are mostly\n"
            "             numerical (as assigned by digicam)\n"
            "             The '.jpg' is automatically added to the end of the name.  If the\n"
            "             destination name already exists, a letter or digit is added to \n"
-           "             the end of the name to make it uniqe.\n"
+           "             the end of the name to make it unique.\n"
            "  -nf[format-string]\n"
            "             Same as -n, but rename regardless of original name\n"
            "  -ta<+|->h[:mm[:ss]]\n"
@@ -1008,7 +1021,7 @@ static void Usage (void)
            "             sections into the image. &i will be substituted for the input file\n"
            "             name, and &o (if &o is used). Use quotes around the command string\n"
            "             This is most useful in conjunction with the free ImageMagic tool. \n"
-           "             For example, with My Cannon S100, which suboptimally compresses\n"
+           "             For example, with my Canon S100, which suboptimally compresses\n"
            "             jpegs I can specify\n"
            "                jhead -cmd \"mogrify -quality 80 &i\" *.jpg\n"
            "             to re-compress a lot of images using ImageMagic to half the size,\n" 
@@ -1137,6 +1150,14 @@ int main (int argc, char **argv)
         }else if (!strcmp(arg,"-de")){
             DeleteExif = TRUE;
             DoModify = TRUE;
+        }else if (!strcmp(arg, "-du")){
+            DeleteUnknown = TRUE;
+            DoModify = TRUE;
+        }else if (!strcmp(arg, "-purejpg")){
+            DeleteExif = TRUE;
+            DeleteComments = TRUE;
+            DeleteUnknown = TRUE;
+            DoModify = TRUE;
         }else if (!strcmp(arg,"-ce")){
             EditComment = TRUE;
             DoModify = TRUE;
