@@ -52,10 +52,12 @@ typedef struct {
     float Distance;
     float CCDWidth;
     float ExposureBias;
+    float DigitalZoomRatio;
     int   FocalLength35mmEquiv; // Exif 2.2 tag - usually not present.
     int   Whitebalance;
     int   MeteringMode;
     int   ExposureProgram;
+    int   ExposureMode;
     int   ISOequivalent;
     int   LightSource;
     char  Comments[MAX_COMMENT];
@@ -132,6 +134,7 @@ void DiscardAllButExif(void);
 int ReadJpegFile(const char * FileName, ReadMode_t ReadMode);
 int TrimExifFunc(void);
 int RemoveSectionType(int SectionType);
+int RemoveUnknownSections(void);
 void WriteJpegFile(const char * FileName);
 Section_t * FindSection(int SectionType);
 Section_t * CreateSection(int SectionType, unsigned char * Data, int size);
